from pyreversi.display import display_plateau_and_score
from pyreversi.rules import is_valid_move,score
from pyreversi.display import warning
from pyreversi.plateau import WHITE,BLACK
import re

def ask_move(board,joueur,game):
    while True:
        answer = input(joueur + " , enter your move : ")
        if not re.match("[A-H][1-8]", answer):
            display_plateau_and_score(board,score(WHITE,board), score(BLACK,board),game)
            warning(
                "no valid place - enter [A-H][1-8] format. For example : B3"
            )
        elif not is_valid_move(board, joueur, answer):
            display_plateau_and_score(board, score(WHITE,board), score(BLACK,board),game)
            warning("no valid move")         
        else:
            return answer


"""
pyreversi init
"""


from pyreversi.args import compute_args
from pyreversi.game import play
from pyreversi.plateau import BLACK,WHITE
from pyreversi.update import update
from pyreversi.display import display_rules, warning
from pyreversi.bot import is_bot
import os


def pyreversi():
    """
    pyreversi entry point
    """
    

    args = compute_args()
    if args.update:
        update()
    elif args.rules:
        display_rules()
    else:
        if compute_args().silent and (not is_bot(BLACK) or not is_bot(WHITE)):
            warning("use --silent only for two bots player")
            os.sys.exit(0)
        play()




"""
pyreversi use case
"""
from pyreversi.plateau import (
    CORNERS,
    CLOSE_CORNERS,
    WHITE,
    BLACK,
    opposite_joueur,
)
from pyreversi.rules import (
    is_valid_move,
    score,
    calcul_nouveau_plateau,
    available_moves,
)
from pyreversi.display import debug
import random
import copy


from pyreversi.args import compute_args


def level_bot(joueur):
    if joueur == WHITE:
        level = compute_args().whitebot
    else:
        level = compute_args().blackbot
    return level


def remove_close_corners_from_move(moves,close_corners):
    for move in close_corners:
        if move in moves:
            debug(move + " can offer a corner! Remove it")
            moves.remove(move)

def corner_from_move(moves,corners):
    for move in corners:
        if move in moves:
            debug("corner available!")
            return move
    return ""

def calcul_bot(board, joueur, level):
    debug("compute available moves for bot")
    moves = available_moves(board, joueur)
    corners = copy.deepcopy(CORNERS)
    close_corners = copy.deepcopy(CLOSE_CORNERS)
    safe_moves = liste_safe_moves(board, joueur, moves)
    if not compute_args().fix:
        random.shuffle(moves)
        random.shuffle(corners)
        random.shuffle(close_corners)
    initmoves = copy.deepcopy(moves)
    if level == 0:
        return moves[0]
    if level == 1:
        return compute_best_score_gain(board, joueur, moves)
    if level == 2:
        move = corner_from_move(moves,corners)
        if move:
            return move
        remove_close_corners_from_move(moves,close_corners)
        if not moves:
            debug("no best availables moves for " + joueur+ ", reinit!")
            moves = copy.deepcopy(initmoves)
        return compute_best_score_gain(board, joueur, moves)
    if level == 3:
        move = corner_from_move(moves,corners)
        if move:
            return move
        remove_close_corners_from_move(moves,close_corners)
        if not moves:
            debug("no availables moves after remove_close_corners_from_move" + joueur+ ", reinit!")
            moves = copy.deepcopy(initmoves)        
        return compute_best_score_gain_iterative_1(board, joueur, moves)
    if level == 4:
        move = corner_from_move(moves,corners)
        if move:
            return move
        delete_safe_close_corners(board, joueur, close_corners)
        remove_close_corners_from_move(moves,close_corners)
        if not moves:
            debug("no availables moves after remove_close_corners_from_move" + joueur+ ", rework!")
            add_no_bad_choices_for_close_corner(board, joueur, moves, close_corners)
        if not moves:
            debug("no availables moves after remove_close_corners_from_move and add_no_bad_choices_for_close_corner" + joueur+ ", reinit!")
            moves = copy.deepcopy(initmoves)
        return compute_best_score_gain_iterative_1_considering_protected(board, joueur, moves)
    if level == 5:
        if safe_moves:
            debug("safe move available! Choose in!")
            moves = safe_moves
        else:
            move = corner_from_move(moves,corners)
            if move:
                return move
            delete_safe_close_corners(board, joueur, close_corners)
            remove_close_corners_from_move(moves,close_corners)
            if not moves:
                debug("no availables moves after remove_close_corners_from_move" + joueur+ ", rework!")
                add_no_bad_choices_for_close_corner(board, joueur, moves, close_corners)
            if not moves:
                debug("no availables moves after remove_close_corners_from_move and add_no_bad_choices_for_close_corner" + joueur+ ", reinit!")
                moves = copy.deepcopy(initmoves)
        return compute_best_score_gain_iterative_1_considering_protected(board, joueur, moves)


def add_no_bad_choices_for_close_corner(board, joueur, moves, close_corners):
    debug("we should verify the best of bad choices")    
    for move in close_corners:
        if is_valid_move(
                    board,
                    joueur,
                    move,
                ):
            sandbox = copy.deepcopy(board)
            calcul_nouveau_plateau(
                        sandbox,
                        joueur,
                        move,
                    )
            if move == "A2" or move == "B1" or move == "B2":
                if not is_valid_move(
                            sandbox, opposite_joueur(joueur), "A1"
                        ):
                    moves.append(move)
            if move == "G1" or move == "G2" or move == "H2":
                if not is_valid_move(
                            sandbox, opposite_joueur(joueur), "H1"
                        ):
                    moves.append(move)
            if move == "A7" or move == "B7" or move == "B8":
                if not is_valid_move(
                            sandbox, opposite_joueur(joueur), "A8"
                        ):
                    moves.append(move)
            if move == "H7" or move == "G7" or move == "G8":
                if not is_valid_move(
                            sandbox, opposite_joueur(joueur), "H8"
                        ):
                    moves.append(move)


def compute_best_score_gain_iterative_1_considering_protected(board, joueur, moves):
    debug("compute best score gain with iterative 1 for " + joueur)
    best_move = ""
    best_gain = -65
    for move in moves:
        sandbox = copy.deepcopy(board)
        calcul_nouveau_plateau(
                sandbox,
                joueur,
                move,
            )
        gain = score_with_protected(joueur, sandbox) - score_with_protected(joueur, board)
        debug("compute potential score protected gain for " + opposite_joueur(joueur) + " if " + move)    
        futur_moves = available_moves(
                sandbox, opposite_joueur(joueur)
            )
        if not futur_moves:
            gain = gain - 0
        best_down_gain = 0
        for futur_move in futur_moves:
            futur_sandbox = copy.deepcopy(sandbox)
            calcul_nouveau_plateau(
                    futur_sandbox,
                    opposite_joueur(joueur),
                    futur_move,
                )
            down_gain = score_with_protected(
                    opposite_joueur(joueur), futur_sandbox
                ) - score_with_protected(opposite_joueur(joueur), sandbox)
            if down_gain > best_down_gain:
                best_down_gain = down_gain
        gain = gain - best_down_gain
        debug(move + " gives " + str(gain) + " pts")
        if gain > best_gain:
            best_move = move
            best_gain = gain
    debug(
            "best move : "
            + best_move
            + " with "
            + str(best_gain)
            + " pts"
        )
        
    return best_move

def compute_best_score_gain_iterative_1(board, joueur, moves):
    debug("compute best score gain with iterative 1 for " + joueur)
    best_move = ""
    best_gain = -65
    for move in moves:
        sandbox = copy.deepcopy(board)
        calcul_nouveau_plateau(
                sandbox,
                joueur,
                move,
            )
        gain = score(joueur, sandbox) - score(joueur, board)
        debug("compute potential score gain for " + opposite_joueur(joueur) + " if " + move)    
        futur_moves = available_moves(
                sandbox, opposite_joueur(joueur)
            )
        if not futur_moves:
            gain = gain - 0
        best_down_gain = 0
        for futur_move in futur_moves:
            futur_sandbox = copy.deepcopy(sandbox)
            calcul_nouveau_plateau(
                    futur_sandbox,
                    opposite_joueur(joueur),
                    futur_move,
                )
            down_gain = score(
                    opposite_joueur(joueur), futur_sandbox
                ) - score(opposite_joueur(joueur), sandbox)
            if down_gain > best_down_gain:
                best_down_gain = down_gain
        gain = gain - best_down_gain
        debug(move + " gives " + str(gain) + " pts")
        if gain > best_gain:
            best_move = move
            best_gain = gain
    debug(
            "best move : "
            + best_move
            + " with "
            + str(best_gain)
            + " pts"
        )
        
    return best_move

def compute_best_score_gain(board, joueur, moves):
    debug("compute best score gain for " + joueur)
    best_move = ""
    best_gain = 0
    for move in moves:
        sandbox = copy.deepcopy(board)
        calcul_nouveau_plateau(
                sandbox,
                joueur,
                move,
            )
        gain = score(joueur, sandbox) - score(joueur, board)
        debug(move + " gives " + str(gain) + " pts")
        if gain > best_gain:
            best_move = move
            best_gain = gain
    debug(
            "best move : "
            + best_move
            + " with "
            + str(best_gain)
            + " pts"
        )
        
    return best_move


def delete_safe_close_corners(board, joueur, close_corners):
    debug("delete_safe_close_corners for " + joueur)
    if board[0][0] == joueur:
        close_corners.remove("A2")
        close_corners.remove("B2")
        close_corners.remove("B1")
    if board[0][7] == joueur:
        close_corners.remove("G1")
        close_corners.remove("G2")
        close_corners.remove("H2")
    if board[7][0] == joueur:
        close_corners.remove("A7")
        close_corners.remove("B7")
        close_corners.remove("B8")
    if board[7][7] == joueur:
        close_corners.remove("G7")
        close_corners.remove("H7")
        close_corners.remove("G8")
    debug("close corners not safe for " + joueur+ " : " + str(close_corners))

def liste_safe_moves(board, joueur, moves):
    safe_moves = []
    debug("search safe moves for "+ joueur)
    for move in moves:
        sandbox = copy.deepcopy(board)
        calcul_nouveau_plateau(
            sandbox,
            joueur,
            move,
        )
        if is_safe_place(
            sandbox,
            joueur,
            int(move[1]) - 1,
            ord(move[0].lower()) - 96 - 1,
        ):
            safe_moves.append(move)
            debug(move + " is a safe move")
    if not safe_moves:
        debug("no safe move")        
    return safe_moves

def is_safe_place(board, joueur, line, column):

    safe = True
    for l in range(line, 8):
        for c in range(min(column + (l - line), 7), -1, -1):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    safe = True
    for l in range(line, 8):
        for c in range(max(column - (l - line), 0), 8):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    safe = True
    for l in range(line, -1, -1):
        for c in range(min(column + (line - l), 7), -1, -1):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    safe = True
    for l in range(line, -1, -1):
        for c in range(max(column - (line - l), 0), 8):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    safe = True
    for c in range(column, 8):
        for l in range(min(line + (c - column), 7), -1, -1):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    safe = True
    for c in range(column, 8):
        for l in range(max(line - (c - column), 0), 8):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    safe = True
    for c in range(column, -1, -1):
        for l in range(min(line + (column - c), 7), -1, -1):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    safe = True
    for c in range(column, -1, -1):
        for l in range(max(line - (column - c), 0), 8):
            if board[l][c] != joueur:
                safe = False
    if safe:
        return True

    return False


def score_with_protected(joueur, board):
    score = 0
    for line in range(0, 8):
        for column in range(0, 8):
            if board[line][column] == joueur:
                if is_safe_place(board, joueur, line, column):
                    score = score + 2
                else:
                    score = score + 1
    return score


def is_bot(joueur):
    return (joueur == WHITE and compute_args().whitebot > -1) or (
        joueur == BLACK and compute_args().blackbot > -1
    )




from pyreversi.plateau import BLACK, WHITE, VIDEE
from pyreversi.display import debug

def is_valid_move(board, joueur, position):

    column = ord(position[0].lower()) - 96 - 1
    line = int(position[1]) - 1

    if board[int(line)][int(column)] != VIDEE:
        return False
    if (
        not is_one_in(board, joueur, line, column, 1, 0)
        and not is_one_in(board, joueur, line, column, 1, 1)
        and not is_one_in(board, joueur, line, column, 0, 1)
        and not is_one_in(board, joueur, line, column, -1, 1)
        and not is_one_in(board, joueur, line, column, -1, 0)
        and not is_one_in(board, joueur, line, column, -1, -1)
        and not is_one_in(board, joueur, line, column, 0, -1)
        and not is_one_in(board, joueur, line, column, 1, -1)
    ):
        return False
    return True


def is_one_in(board, joueur, line, column, dirline, dircol):
    check_line = line
    check_column = column
    if joueur == WHITE:
        joueur_against = BLACK
    else:
        joueur_against = WHITE
    check_one = False
    while True:
        check_line = check_line + dirline
        check_column = check_column + dircol

        if (
            check_column > 7
            or check_line > 7
            or check_column < 0
            or check_line < 0
            or board[int(check_line)][int(check_column)] == VIDEE
        ):
            return False
        if board[int(check_line)][int(check_column)] == joueur:
            return check_one
        if (
            board[int(check_line)][int(check_column)]
            == joueur_against
        ):
            check_one = True


def return_in(board, joueur, line, column, dirline, dircol):
    check_line = line
    check_column = column
    if joueur == WHITE:
        joueur_against = BLACK
    else:
        joueur_against = WHITE
    check_one = False
    while True:
        check_line = check_line + dirline
        check_column = check_column + dircol

        if check_column > 7 or check_line > 7:
            return
        if check_column < 0 or check_line < 0:
            return
        if board[int(check_line)][int(check_column)] == VIDEE:
            return
        if (
            check_one
            and board[int(check_line)][int(check_column)]
            == joueur
        ):
            return
        if (
            board[int(check_line)][int(check_column)]
            == joueur_against
        ):
            board[int(check_line)][int(check_column)] = joueur
            check_one = True

    # N
    # E
    # W
    # NE
    # SE
    # SO
    # NO
    #
    #


def score(joueur, board):
    return sum([i.count(joueur) for i in board])


def calcul_nouveau_plateau(board, joueur, position):

    column = ord(position[0].lower()) - 96 - 1
    line = int(position[1]) - 1

    board[int(line)][int(column)] = joueur
    if is_one_in(board, joueur, line, column, 1, 0):
        return_in(board, joueur, line, column, 1, 0)
    if is_one_in(board, joueur, line, column, 1, 1):
        return_in(board, joueur, line, column, 1, 1)
    if is_one_in(board, joueur, line, column, 0, 1):
        return_in(board, joueur, line, column, 0, 1)
    if is_one_in(board, joueur, line, column, -1, 1):
        return_in(board, joueur, line, column, -1, 1)
    if is_one_in(board, joueur, line, column, -1, 0):
        return_in(board, joueur, line, column, -1, 0)
    if is_one_in(board, joueur, line, column, -1, -1):
        return_in(board, joueur, line, column, -1, -1)
    if is_one_in(board, joueur, line, column, 0, -1):
        return_in(board, joueur, line, column, 0, -1)
    if is_one_in(board, joueur, line, column, 1, -1):
        return_in(board, joueur, line, column, 1, -1)


def available_moves(board, joueur):
    moves = []
    for i in range(0, 8):
        for j in range(0, 8):
            move = chr(65 + i) + str(j + 1)
            if is_valid_move(board, joueur, move):
                moves.append(move)
    debug("available_moves for " + joueur +" : " + str(moves))                 
    return moves

def can_play(board,joueur):
    if available_moves(board, joueur) ==[]:
        return False
    return True  

def end_game(board):
    if not can_play(board, WHITE) and not can_play(board, BLACK):
        return True
    return False  





CORNERS = ["A1", "A8", "H1", "H8"]
BLACK = "\U000026AB"
WHITE = "\U000026AA"
VIDEE = " "

INIT_PLATEAU = [
    [VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE],
    [VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE],
    [VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE],
    [VIDEE, VIDEE, VIDEE, WHITE, BLACK, VIDEE, VIDEE, VIDEE],
    [VIDEE, VIDEE, VIDEE, BLACK, WHITE, VIDEE, VIDEE, VIDEE],
    [VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE],
    [VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE],
    [VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE, VIDEE],
]
CLOSE_CORNERS = [
    "A2",
    "B2",
    "B1",
    "G1",
    "G2",
    "H2",
    "A7",
    "B7",
    "B8",
    "G8",
    "G7",
    "H7",
]


def opposite_joueur(joueur):
    if joueur == BLACK:
        joueur_against = WHITE
    else:
        joueur_against = BLACK
    return joueur_against


"""
pyreversi use case
"""
from pyreversi.plateau import BLACK, WHITE, VIDEE
from columnar import columnar
from click import style
import os


from pyreversi.args import compute_args


def clear():
    if not compute_args().verbose and not compute_args().silent:
        command = "clear"
        if os.name in (
            "nt",
            "dos",
        ):  # If Machine is running on Windows, use cls
            command = "cls"
        os.system(command)


def display_plateau_and_score(plateau, scorew, scoreb,game):
    if not compute_args().silent:
        clear()
        data = []
        headers = ["", "A", "B", "C", "D", "E", "F", "G", "H"]
        for i in range(1, 9):
            ligne = []
            ligne.append(i)
            for j in range(0, 8):
                ligne.append(plateau[i - 1][j])
            data.append(ligne)
        patterns = [
            (WHITE, lambda text: style(text, bg="green")),
            (BLACK, lambda text: style(text, bg="green")),
            (VIDEE, lambda text: style(text, bg="green")),
        ]
        try:
            table = columnar(
                data,
                headers,
                no_borders=False,
                wrap_max=0,
                patterns=patterns,
                justify="c",
                max_column_width=2,
            )
        except Exception:
            table = columnar(
                data,
                headers,
                no_borders=False,
                wrap_max=0,
                #patterns=patterns,
                justify="c",
                max_column_width=2,
            )                
        if compute_args().games!=0:
            info("BATCH MODE : game " + str(game) + " / " + str(compute_args().games))
        info(table)
        if compute_args().whitebot != -1:
            pw = WHITE + " (IA lvl " + str(compute_args().whitebot) + ")"
        else:
            pw = WHITE
        if compute_args().blackbot != -1:
            pb = BLACK + " (IA lvl " + str(compute_args().blackbot) + ")"
        else:
            pb = BLACK
        info(
            "score : "
            + pb
            + " "
            + str(scoreb).rjust(2, "0")
            + " - "
            + str(scorew).rjust(2, "0")
            + " "
            + pw
        )


def display_endgame(board, scorew, scoreb):
    info("game over!")
    if scorew > scoreb:
        info(WHITE + " wins !")
    elif scorew < scoreb:
        info(BLACK + " wins !")
    else:
        info("draw match !")


def display_cant_play(joueur):
    inputauto(joueur + " , no move available, press any key")


def display_bot_move(joueur, position):
    info(joueur + " , plays move : " + position)
    inputauto("press any key")


def display_rules():
    warning("RULES OF REVERSI : ")
    warning(
        "- Reversi is a two-player strategy game played on an 8x8 board using discs that are colored white on one side and black on the other. One player plays the discs black side up while his opponent plays the discs white side up."
    )
    warning(
        "- The object of the game is to place your discs on the board so as to outflank your opponent's discs, flipping them over to your color. The player who has the most discs on the board at the end of the game wins."
    )
    warning(
        "- Note: For convenience, board positions are denoted by a letter representing the column (A through H) and a number representing the row (1 through 8). For example, the top-left square on the board is referred to as A1 while the square to the right of it is referred to as B1."
    )
    warning(
        "- Every game starts with four discs placed in the center of the board"
    )
    warning("- Black begins")
    warning(
        "- Players take turns making moves. A move consists of a player placing a disc of his color on the board. The disc must be placed so as to outflank one or more opponent discs, which are then flipped over to the current player's color."
    )
    warning(
        "- Outflanking your opponent means to place your disc such that it traps one or more of your opponent's discs between another disc of your color along a horizontal, vertical or diagonal line through the board square "
    )
    warning(
        "- If a player cannot make a legal move, he forfeits his turn and the other player moves again (this is also known as passing a turn). Note that a player may not forfeit his turn voluntarily. If a player can make a legal move on his turn, he must do so."
    )
    warning(
        "- The game ends when neither player can make a legal move. This includes when there are no more empty squares on the board or if one player has flipped over all of his opponent's discs (a situation commonly known as a wipeout)."
    )
    warning(
        "- The player with the most discs of his color on the board at the end of the game wins. The game is a draw if both players have the same number of discs."
    )
    warning(
        "- When making a move, you may outflank your opponent's discs in more than one direction. All outflanked discs are flipped."
    )
    warning(
        "source : https://documentation.help/Reversi-Rules/rules.htm"
    )


def debug(phrase):
    if compute_args().verbose:
        print("debug : " + phrase)

def info(phrase):
    if not compute_args().silent:
        print(phrase)

def warning(phrase):
    print(phrase)  

def inputauto(phrase):
    if not compute_args().auto and not compute_args().silent:    
        input(phrase)      



"""
pyreversi use case
"""
from pyreversi.human import ask_move
from pyreversi.display import (
    display_plateau_and_score,
    display_endgame,
    display_cant_play,
    display_bot_move,
    debug,
    warning,
    clear
)
from pyreversi.plateau import (
    BLACK,
    WHITE,
    INIT_PLATEAU,
    opposite_joueur,
)
from pyreversi.rules import (
    end_game,
    score,
    calcul_nouveau_plateau,
    can_play,
    end_game,
)
from pyreversi.bot import calcul_bot, level_bot, is_bot
import os
import copy

from pyreversi.args import compute_args


def play():

    board = copy.deepcopy(INIT_PLATEAU)
    joueur = None
    game=1
    winw=0
    winb=0
    draw=0
    totw=0
    totb=0
    while True:
        joueur = opposite_joueur(joueur)
        display_plateau_and_score(board,score(WHITE,board), score(BLACK,board),game)
        debug("compute if endgame")
        if end_game(board):
            display_endgame(board, score(WHITE,board), score(BLACK,board))
            if compute_args().games == 0:
                os.sys.exit(0)
            else:
                totw=totw+score(WHITE,board)
                totb=totb+score(BLACK,board)
                if score(WHITE,board)>score(BLACK,board):
                    winw=winw+1
                elif score(WHITE,board)<score(BLACK,board):
                    winb=winb+1
                else:
                    draw=draw+1       
                if game == compute_args().games:
                    clear()
                    if compute_args().whitebot != -1:
                        pw = WHITE + " (IA lvl " + str(compute_args().whitebot) + ")"
                    else:
                        pw = WHITE
                    if compute_args().blackbot != -1:
                        pb = BLACK + " (IA lvl " + str(compute_args().blackbot) + ")"
                    else:
                        pb = BLACK 
                    warning("FIN BATCH : " + pb + " - " + pw)
                    warning("Victories " + BLACK + " : " + str(winb))
                    warning("Victories " + WHITE + " : " + str(winw))
                    warning("Draws : " + str(draw))
                    warning("Points " + BLACK + " : " + str(totb))
                    warning("Points " + WHITE + " : " + str(totw))
                    os.sys.exit(0)
                else:
                    game=game+1
                    board = copy.deepcopy(INIT_PLATEAU)
                    joueur = None
                    joueur = opposite_joueur(joueur)
                    display_plateau_and_score(board,score(WHITE,board), score(BLACK,board),game)                                        
        debug("compute if " + joueur + " can play")    
        if not can_play(board, joueur):
            display_cant_play(joueur)
            joueur = opposite_joueur(joueur)

        if not is_bot(joueur):
            position = ask_move(board,joueur,game)
        else:
            position = calcul_bot(board, joueur, level_bot(joueur))
            display_bot_move(joueur, position)
        calcul_nouveau_plateau(board, joueur, position)


